/*
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2004-2007 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <boost/assign/list_of.hpp>

#include "intent_selector.h"

using namespace TkResolver;
using namespace Experimenter;



IntentSelector:: IntentSelector(Db::Database * database, unsigned trace_count)
  : _store( Gtk::ListStore::create(_columns) )
{
  set_model(_store);
  append_column("Code", _columns.code);
  append_column("Description", _columns.description);
  
  fill_list(database, trace_count);
}

void
IntentSelector:: fill_list(Db::Database * database, unsigned trace_count) {
  Db::IntentInfoList info( database->get_intent_options(trace_count) );
  String default_intent( database->choose_intent(trace_count) );
  
  for (Db::IntentInfoList::iterator iter( info.begin() );
       iter != info.end();
       ++iter)
    {
      Gtk::ListStore::iterator row( _store->append() );
      
      row->set_value(_columns.code, wstring_to_ustring(iter->first));
      row->set_value(_columns.description, wstring_to_ustring(iter->second));
      
      // default
      if (iter->first == default_intent)
	get_selection()->select(row);
    }
}

/*
bool
IntentSelector:: default_row_foreach(const TreeModel::Path &path, const TreeModel::iterator &iter, const Db::Code &default)
{
  if (Db::Code(iter->get_value(_columns.code)) == default)
    {
      
    }
}

void
IntentSelector:: set_default(Db::Database * database, unsigned trace_count) {
  Db::Code default( database->choose_intent(trace_count) );
  
  Gtk::TreeModel::SlotForeachPathAndIter slot( sigc::bind<Db::Code>(sig::mem_fun(*this, &IntentSelector::default_row_foreach), default) );

}
*/

Db::Code
IntentSelector:: get_intent_code()
{
  return get_selection()->get_selected()->get_value(_columns.code);
}
