/* -*- c++ -*-
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2008 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "libtkcommon/debug.h"

#include "libtkquery/expression.h"

using namespace TkResolver::Query;

static const TkResolver::String obrkt("(");
static const TkResolver::String cbrkt(")");
static const TkResolver::String space(" ");

TkResolver::String
Boolean:: build_query(QueryTables &tables)
{
  //WDEBUG_OUT << L"building Boolean" << std::endl;
  if (_val)
    return "1";
  else
    return "0";
}

TkResolver::String
Integer:: build_query(QueryTables &tables)
{
  //WDEBUG_OUT << L"building Integer (" << _val << ")" << std::endl;
  return _val;
}


TkResolver::String
Real:: build_query(QueryTables &tables)
{
  //WDEBUG_OUT << L"building Real" << std::endl;
  return _val;
}

TkResolver::String
String:: build_query(QueryTables &tables)
{
  //WDEBUG_OUT << L"building String" << std::endl;
  return TkResolver::String("'") + _val + TkResolver::String("'");
}

TkResolver::String
Identifier:: build_query(QueryTables &tables)
{
  //WDEBUG_OUT << L"building Identifier (name: '" << _name << "')" << std::endl;
  TableAlias property_value( tables.add_property_value_table(_name) );
  TableAlias property( tables.add_table("property") );
  TableAlias property_type( tables.add_table("property_type") );
  TableAlias experiment_property( tables.add_table("experiment_property") );
  
  tables.add_link_clause(experiment_property + TkResolver::String(".experiment_id = e.experiment_id"));
  tables.add_link_clause(property + TkResolver::String(".property_id = ") + experiment_property + TkResolver::String(".property_id"));
  tables.add_link_clause(property_value + TkResolver::String(".property_id = ") + property + TkResolver::String(".property_id"));
  tables.add_link_clause(property_type + TkResolver::String(".property_type_id = ") + property + TkResolver::String(".property_type_id"));
  tables.add_link_clause(property_type + TkResolver::String(".name = '") + _name + TkResolver::String("'"));
  
  return property_value + TkResolver::String(".property_value");
}


TkResolver::String
Binary:: build_query(QueryTables &tables)
{
  //WDEBUG_OUT << L"building Binary" << std::endl;
  return obrkt
    + _val1->build_query(tables)
    + space
    + _op->get_op()
    + space
    + _val2->build_query(tables)
    + cbrkt;
}

TkResolver::String
Unary:: build_query(QueryTables &tables)
{
  //WDEBUG_OUT << L"building Unary" << std::endl;
  return obrkt + _val->build_query(tables) + TkResolver::String(" = 0)");
}



TkResolver::String
Bracketed:: build_query(QueryTables &tables)
{
  //WDEBUG_OUT << L"building Bracketed" << std::endl;
  return obrkt + _expr->build_query(tables) + cbrkt;
}

TkResolver::String
Logical:: build_query(QueryTables &tables)
{
  //WDEBUG_OUT << L"building Logical" << std::endl;
  TkResolver::String op;
  if (_lop->get_op() == "&")
    op = "and";
  else
    op = "or";
  
  return _expr1->build_query(tables) + space + op + space + _expr2->build_query(tables);
}
