/* -*- c++ -*-
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2008 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <errno.h>

#include "libtkcommon/exceptions.h"
#include "libtkcommon/debug.h"

#include "libtkquery/query.h"
#include "libtkquery/tkquery-parser.h"

using namespace TkResolver;
using namespace TkResolver::Query;

extern FILE *yyin;
extern int yy_flex_debug;
Expression *expr_glbl;

// Mandatory error function
void yy::parser::error(const yy::parser::location_type& yylloc, const std::string& message)
{
  std::cerr << yylloc << ": " << message << std::endl;
}


static Expression *
parse_file(const TkResolver::String &filename)
{
  //  yy_flex_debug = 1;

  yyin = fopen(std::string(filename).c_str(), "r");
  if (yyin == NULL)
    throw UnixError(std::string("Could not open query file '") + std::string(filename) + "'", errno);
  
  yy::parser *parser( new yy::parser() );
  //  parser->set_debug_level(1);
  int err( parser->parse() );

  delete parser;
  fclose(yyin);
  
  if (err)
    throw std::runtime_error(std::string("Could not parse query file '") + std::string(filename) + "'");
  
  return expr_glbl;
}

TkResolver::String
TkResolver::Query:: build_query(Db::Database *database, const TkResolver::String &filename)
{
  Expression *expr( parse_file(filename) );
  //WDEBUG_OUT << L"File parsed" << std::endl;
  
  QueryTables tables(database);
  TkResolver::String expression( expr->build_query(tables) );
  tables.add_link_clause(TkResolver::String("(") + expression + TkResolver::String(")"));
  TkResolver::String sql(tables.get_sql());
  
  delete expr;
  //WDEBUG_OUT << L"sql: " << sql << std::endl;
  return sql;
}
 
Db::IdList
TkResolver::Query:: get_experiments(Db::Database *database, const TkResolver::String &filename)
{
  TkResolver::String sql( build_query(database, filename) );
  Db::DataSetPtr results(database->execute_query(sql));
  
  bool valid( (*results)->next() );
  if (!valid)
    throw std::runtime_error("no experiments");
  
  Db::IdList ids;
  for (;
       !(*results)->isAfterLast();
       (*results)->next())
    {
      ids.push_back((*results)->getLong(1));
    }
  
  return ids;
}
