/* -*- c++ -*-
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2008 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "libtkcommon/debug.h"

#include "libtkquery/query_tables.h"

using namespace TkResolver;
using namespace TkResolver::Query;


QueryTables:: QueryTables(Db::Database *database)
  : _next_table_no(0), _database(database)
{
}

TableAlias
QueryTables:: add_table(const TableName &name)
{
  TableId id( _next_table_no++ );
  char c( static_cast<char>(*name.begin()) );
  TableAlias alias( TkResolver::String(c) + TkResolver::String(id) );
  
  _tables[alias] = name;
  
  return alias;
}

TableAlias
QueryTables:: add_property_value_table(const TkResolver::String &identifier)
{
  TkResolver::String table_name;
  _database->property_type_get_details(identifier, 0, 0, &table_name);
  
  return add_table(table_name);
}

void
QueryTables:: add_link_clause(const TkResolver::String &clause)
{
  _link_clauses.push_back(clause);
}

TkResolver::String
QueryTables:: get_sql() const
{
  TkResolver::String sql = "select e.experiment_id from experiment e";
  
  for (TableMap::const_iterator iter( _tables.begin() );
       iter != _tables.end();
       ++iter)
    {
      sql += TkResolver::String(", ");
      sql += iter->second;
      sql += TkResolver::String(" as ");
      sql += iter->first;
    }
  
  sql += TkResolver::String(" where ");
  for (TkResolver::StringList::const_iterator iter( _link_clauses.begin() );
       iter != _link_clauses.end();
       ++iter)
    {
      if (iter != _link_clauses.begin())
	sql += TkResolver::String(" and ");
      sql += *iter;
    }
  
  return sql;
}
